@@ -247,6 +247,19 @@ sub _emit_pair {
     return '{' . _emit( $n->{key} ) . '=>' . $value . '}'
 }
 
+sub _emit_complex {
+    my $n = shift;
+    $n =~ s/i/*Math::Complex::i/;
+    "($n)";
+}
+
+sub _emit_num {
+    my $n = shift;
+    return '$Math::Complex::Inf' if $n eq 'Inf';
+    return '($Math::Complex::Inf - $Math::Complex::Inf)' if $n eq 'NaN';
+    "$n";
+}
+
 sub _emit {
     my $n = $_[0];
     #die "_emit: ", Dumper( $n );
@@ -289,9 +302,12 @@ sub _emit {
     return $n->{int}
         if exists $n->{int};
 
-    return $n->{num}
+    return _emit_num( $n->{num} )
         if exists $n->{num};
 
+    return _emit_complex( $n->{complex} )
+        if exists $n->{complex};
+
     return _emit_pair( $n->{pair} )
         if exists $n->{pair};
 
@@ -826,6 +842,15 @@ sub default {
         # TODO - other builtins
         my $subname = $n->{sub}{bareword};
         if ( $subname ) {
+            if ($subname eq 'next') {
+                my $param = $n->{param} ? _emit( $n->{param} ) : '';
+                return " next $param ";
+            }
+            if ($subname eq 'redo') {
+                my $param = $n->{param} ? _emit( $n->{param} ) : '';
+                return " redo $param ";
+            }
+
             if ($subname eq 'defined') {
                 my $param = _emit( $n->{param} );
                 # when testing defined-ness of $!, it is testing the emptiness of $@ in perl5.
@@ -833,12 +858,22 @@ sub default {
                 return " (defined $param )";
             }
 
-            if (   $subname eq 'any' || $subname eq 'all'  
-                || $subname eq 'substr' || $subname eq 'split' || $subname eq 'die' || $subname eq 'return' 
-                || $subname eq 'push' || $subname eq 'pop' || $subname eq 'shift' || $subname eq 'join' 
-                || $subname eq 'index' || $subname eq 'undef' || $subname eq 'rand' || $subname eq 'int' 
-                || $subname eq 'splice' || $subname eq 'keys' || $subname eq 'values' || $subname eq 'sort' 
-                || $subname eq 'chomp' || $subname eq 'lc' || $subname eq 'abs' || $subname eq 'sleep' 
+            if ($subname eq 'readline') {
+                my $param = _emit( $n->{param} );
+                return "Pugs::Runtime::Perl6::IO::readline( $param )";
+            }
+
+            if ($subname eq 'pi') {
+                return "Math::Complex::pi()";
+            }
+
+            if (   $subname eq 'any'    || $subname eq 'all'  
+                || $subname eq 'substr' || $subname eq 'split'  || $subname eq 'die'    || $subname eq 'return' 
+                || $subname eq 'push'   || $subname eq 'pop'    || $subname eq 'shift'  || $subname eq 'join' 
+                || $subname eq 'index'  || $subname eq 'undef'  || $subname eq 'rand'   || $subname eq 'int' 
+                || $subname eq 'splice' || $subname eq 'keys'   || $subname eq 'values' || $subname eq 'sort' 
+                || $subname eq 'chomp'  || $subname eq 'lc'     || $subname eq 'abs'    || $subname eq 'sleep' 
+                || $subname eq 'unlink' || $subname eq 'close'  || $subname eq 'sqrt'
                 ) 
             {
                 return $subname . emit_parenthesis( $n->{param} );
@@ -914,15 +949,27 @@ sub default {
         if ( $n->{method}{dot_bareword} eq 'print' ||
              $n->{method}{dot_bareword} eq 'warn' ) {
             my $s = _emit( $n->{self} );
-            if ( $s eq Pugs::Runtime::Common::mangle_var('$*ERR') ) {
-                return " print STDERR '', " . _emit( $n->{param} );
+            if ( exists $n->{self}{scalar} && $n->{self}{scalar} eq '$*ERR' ) {
+                return " print STDERR " . ( $n->{param} ? _emit( $n->{param} ) : "''" );
+            }
+            if ( exists $n->{self}{scalar} && $n->{self}{scalar} eq '$*OUT' ) {
+                return " print STDOUT " . ( $n->{param} ? _emit( $n->{param} ) : "''" );
+            }
+            if ( exists $n->{self}{scalar} && $n->{self}{scalar} ) {
+                return "$s->print" . emit_parenthesis( $n->{param} );
             }
             return " print '', $s";
         }
         if ( $n->{method}{dot_bareword} eq 'say' ) {
             my $s = _emit( $n->{self} );
-            if ( $s eq Pugs::Runtime::Common::mangle_var('$*ERR') ) {
-                return " print STDERR '', " . _emit( $n->{param} ) . ', "\n"';
+            if ( exists $n->{self}{scalar} && $n->{self}{scalar} eq '$*OUT' ) {
+                return " print STDOUT " . ( $n->{param} ? _emit( $n->{param} ) : "''" ) . ', "\n"';
+            }
+            if ( exists $n->{self}{scalar} && $n->{self}{scalar} eq '$*ERR' ) {
+                return " print STDERR " . ( $n->{param} ? _emit( $n->{param} ) : "''" ) . ', "\n"';
+            }
+            if ( exists $n->{self}{scalar} && $n->{self}{scalar} ) {
+                return "$s->say" . emit_parenthesis( $n->{param} );
             }
             return " print '', $s" . ', "\n"';
         }
@@ -1103,11 +1150,15 @@ sub statement {
         if ( $n->{else} ) {
             $ret .= 'else '. emit_block( $n->{else} ) . "\n";
         }
+        else {
+            $ret .= "else { () }\n";
+        }
         return $ret;
     }
 
     if ( $n->{statement} eq 'do' ) {
-        return 'do ' . emit_block( $n->{exp1} );
+        # double braces in 'do' allow for 'next/last'
+        return 'do {{ ' . emit_block( $n->{exp1} ) . ' }}';
     }
     if ( $n->{statement} eq 'given' ) {
         return  'for (1) { local $_ = ' . _emit( $n->{exp1} ) . '; ' .
@@ -1280,6 +1331,7 @@ sub term {
                 our \@EXPORT;
                 bool->import();  # True, False
                 use Quantum::Superpositions;
+                use Math::Complex ();
                 $attributes ";
 
         return ref( $n->{block} ) && exists $n->{block}{bare_block}
@@ -1623,6 +1675,14 @@ sub infix {
                       'chain' => [ $n->{exp1}, 'eq', $n->{exp2} ]
                 } );
         }
+        if (   exists $n->{exp2}{pair} 
+            && $n->{exp2}{pair}{key}{single_quoted} eq 'e'
+            && $n->{exp2}{pair}{value}{num} eq 1
+            ) 
+        {
+            #  $filename ~~ :e
+            return "( -e " . _emit( $n->{exp1} ) . ")";
+        }
         return _emit( $n->{exp1} ) . ' =~ (ref' . emit_parenthesis( $n->{exp2} ).' eq "Regexp" '.
             ' ? '._emit($n->{exp2}).
             ' : quotemeta'.emit_parenthesis($n->{exp2}).
@@ -1792,11 +1852,6 @@ sub postcircumfix {
     if ( $n->{op1} eq '[' &&
          $n->{op2} eq ']' ) {
 
-        if ( ! exists $n->{exp2} ) {
-            # $array[]
-            return '@{ ' . _emit( $n->{exp1} ) . ' }';
-        }
-
         if ( ! defined $n->{exp2}
             && exists $n->{exp1}{array}
             )
@@ -1805,6 +1860,11 @@ sub postcircumfix {
             return _emit( $n->{exp1} );
         }
 
+        if ( ! defined $n->{exp2} ) {
+            # $array[]
+            return '@{ ' . _emit( $n->{exp1} ) . ' }';
+        }
+
         # avoid p5 warning - "@a[1] better written as $a[1]"
         if (   (  exists $n->{exp2}{int}
                || exists $n->{exp2}{scalar}
