@@ -132,7 +132,7 @@ sub rx_body {
                 <?ws>? $<invocant> := <Pugs::Grammar::Term.parse> <?ws>? \:
                 [
                     <?ws> 
-                    <Pugs::Grammar::Expression.parse('allow_semicolon', 1)> <?ws>? 
+                    <Pugs::Grammar::Expression.parse('no_comma', 1)> <?ws>? 
                     ')'
                     { return {
                         op1      => "(",
@@ -663,18 +663,34 @@ sub recompile {
                 <Pugs::Grammar::Quote.q>
                 { return $/{'Pugs::Grammar::Quote.q'}->() }
             ^ ),
+        q(WHAT) => Pugs::Compiler::Token->compile( q^
+                <?ws> <Pugs::Grammar::Expression.parse('no_comma',1)> 
+                    { return {
+                        op1      => 'call',
+                        param    => $_[0]{'Pugs::Grammar::Expression.parse'}->(),
+                        sub      => {
+                                        'bareword' => 'WHAT',
+                                    },
+                    } }
+            ^ ),
         q() => Pugs::Compiler::Token->compile( q^
-                ### num/int
+                ### num/int/complex
                 \d+ 
                 [
                     \.\d+
                     [ <[Ee]> <[+-]>? \d+ ]?
-                    { return { num => $() ,} } 
+                    [  i  { return { complex => $() ,} } 
+                    |     { return { num     => $() ,} } 
+                    ]
                 |
                     <[Ee]> <[+-]>? \d+ 
-                    { return { num => $() ,} } 
+                    [  i  { return { complex => $() ,} } 
+                    |     { return { num     => $() ,} } 
+                    ]
                 |
-                    { return { int => $() ,} } 
+                    [  i  { return { complex => $() ,} } 
+                    |     { return { int     => $() ,} } 
+                    ]
                 ]
             |
                 <Pugs::Grammar::Perl6.sub_decl>
